@@ -336,10 +336,11 @@ static int connection_handle_read(server *srv, connection *con) {
 	len = recv(con->fd, mem, mem_len, 0);
 #else /* __WIN32 */
 	if (ioctl(con->fd, FIONREAD, &toread) || toread == 0 || toread <= 4*1024) {
-		if (toread > MAX_READ_LIMIT) toread = MAX_READ_LIMIT;
-	} else {
 		toread = 4096;
 	}
+	else if (toread > MAX_READ_LIMIT) {
+		toread = MAX_READ_LIMIT;
+	}
 	chunkqueue_get_memory(con->read_queue, &mem, &mem_len, 0, toread);
 
 	len = read(con->fd, mem, mem_len);
@@ -688,7 +689,10 @@ connection *connection_init(server *srv) {
 	con->write_queue = chunkqueue_init();
 	con->read_queue = chunkqueue_init();
 	con->request_content_queue = chunkqueue_init();
-	chunkqueue_set_tempdirs(con->request_content_queue, srv->srvconf.upload_tempdirs);
+	chunkqueue_set_tempdirs(
+		con->request_content_queue,
+		srv->srvconf.upload_tempdirs,
+		srv->srvconf.upload_temp_file_size);
 
 	con->request.headers      = array_init();
 	con->response.headers     = array_init();
@@ -931,7 +935,7 @@ static int connection_handle_read_state(server *srv, connection *con)  {
 
 					for ( ; cc; cc = cc->next, j = 0 ) {
 						size_t bblen = buffer_string_length(cc->mem) - cc->offset;
-						const char *bb = c->mem->ptr + cc->offset;
+						const char *bb = cc->mem->ptr + cc->offset;
 
 						for ( ; j < bblen; j++) {
 							ch = bb[j];
@@ -982,7 +986,11 @@ found_header_end:
 		}
 		break;
 	case CON_STATE_READ_POST:
-		if (0 != chunkqueue_steal_with_tempfiles(srv, dst_cq, cq, con->request.content_length - dst_cq->bytes_in )) {
+		if (con->request.content_length <= 64*1024) {
+			/* don't buffer request bodies <= 64k on disk */
+			chunkqueue_steal(dst_cq, cq, con->request.content_length - dst_cq->bytes_in);
+		}
+		else if (0 != chunkqueue_steal_with_tempfiles(srv, dst_cq, cq, con->request.content_length - dst_cq->bytes_in )) {
 			con->http_status = 413; /* Request-Entity too large */
 			con->keep_alive = 0;
 			connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
